@@ -14,19 +14,116 @@
 //! future, as well as generate and broadcast funding transactions handle payment preimages and a
 //! few other things.
 
-use ln::msgs;
-use ln::{PaymentPreimage, PaymentHash, PaymentSecret};
 use chain::keysinterface::SpendableOutputDescriptor;
-use util::ser::{Writeable, Writer, MaybeReadable, Readable, VecReadWrapper, VecWriteWrapper};
+use ln::msgs;
+use ln::msgs::DecodeError;
+use ln::{PaymentPreimage, PaymentHash, PaymentSecret};
+use routing::network_graph::NetworkUpdate;
+use util::ser::{BigSize, FixedLengthReader, Writeable, Writer, MaybeReadable, Readable, VecReadWrapper, VecWriteWrapper};
+use routing::router::RouteHop;
 
 use bitcoin::blockdata::script::Script;
 
 use bitcoin::secp256k1::key::PublicKey;
 
+use io;
 use prelude::*;
 use core::time::Duration;
 use core::ops::Deref;
 
+/// Some information provided on receipt of payment depends on whether the payment received is a
+/// spontaneous payment or a "conventional" lightning payment that's paying an invoice.
+#[derive(Clone, Debug)]
+pub enum PaymentPurpose {
+	/// Information for receiving a payment that we generated an invoice for.
+	InvoicePayment {
+		/// The preimage to the payment_hash, if the payment hash (and secret) were fetched via
+		/// [`ChannelManager::create_inbound_payment`]. If provided, this can be handed directly to
+		/// [`ChannelManager::claim_funds`].
+		///
+		/// [`ChannelManager::create_inbound_payment`]: crate::ln::channelmanager::ChannelManager::create_inbound_payment
+		/// [`ChannelManager::claim_funds`]: crate::ln::channelmanager::ChannelManager::claim_funds
+		payment_preimage: Option<PaymentPreimage>,
+		/// The "payment secret". This authenticates the sender to the recipient, preventing a
+		/// number of deanonymization attacks during the routing process.
+		/// It is provided here for your reference, however its accuracy is enforced directly by
+		/// [`ChannelManager`] using the values you previously provided to
+		/// [`ChannelManager::create_inbound_payment`] or
+		/// [`ChannelManager::create_inbound_payment_for_hash`].
+		///
+		/// [`ChannelManager`]: crate::ln::channelmanager::ChannelManager
+		/// [`ChannelManager::create_inbound_payment`]: crate::ln::channelmanager::ChannelManager::create_inbound_payment
+		/// [`ChannelManager::create_inbound_payment_for_hash`]: crate::ln::channelmanager::ChannelManager::create_inbound_payment_for_hash
+		payment_secret: PaymentSecret,
+		/// This is the `user_payment_id` which was provided to
+		/// [`ChannelManager::create_inbound_payment_for_hash`] or
+		/// [`ChannelManager::create_inbound_payment`]. It has no meaning inside of LDK and is
+		/// simply copied here. It may be used to correlate PaymentReceived events with invoice
+		/// metadata stored elsewhere.
+		///
+		/// [`ChannelManager::create_inbound_payment`]: crate::ln::channelmanager::ChannelManager::create_inbound_payment
+		/// [`ChannelManager::create_inbound_payment_for_hash`]: crate::ln::channelmanager::ChannelManager::create_inbound_payment_for_hash
+		user_payment_id: u64,
+	},
+	/// Because this is a spontaneous payment, the payer generated their own preimage rather than us
+	/// (the payee) providing a preimage.
+	SpontaneousPayment(PaymentPreimage),
+}
+
+#[derive(Clone, Debug, PartialEq)]
+/// The reason the channel was closed. See individual variants more details.
+pub enum ClosureReason {
+	/// Closure generated from receiving a peer error message.
+	///
+	/// Our counterparty may have broadcasted their latest commitment state, and we have
+	/// as well.
+	CounterpartyForceClosed {
+		/// The error which the peer sent us.
+		///
+		/// The string should be sanitized before it is used (e.g emitted to logs
+		/// or printed to stdout). Otherwise, a well crafted error message may exploit
+		/// a security vulnerability in the terminal emulator or the logging subsystem.
+		peer_msg: String,
+	},
+	/// Closure generated from [`ChannelManager::force_close_channel`], called by the user.
+	///
+	/// [`ChannelManager::force_close_channel`]: crate::ln::channelmanager::ChannelManager::force_close_channel.
+	HolderForceClosed,
+	/// The channel was closed after negotiating a cooperative close and we've now broadcasted
+	/// the cooperative close transaction. Note the shutdown may have been initiated by us.
+	//TODO: split between CounterpartyInitiated/LocallyInitiated
+	CooperativeClosure,
+	/// A commitment transaction was confirmed on chain, closing the channel. Most likely this
+	/// commitment transaction came from our counterparty, but it may also have come from
+	/// a copy of our own `ChannelMonitor`.
+	CommitmentTxConfirmed,
+	/// Closure generated from processing an event, likely a HTLC forward/relay/reception.
+	ProcessingError {
+		/// A developer-readable error message which we generated.
+		err: String,
+	},
+	/// The `PeerManager` informed us that we've disconnected from the peer. We close channels
+	/// if the `PeerManager` informed us that it is unlikely we'll be able to connect to the
+	/// peer again in the future or if the peer disconnected before we finished negotiating
+	/// the channel open. The first case may be caused by incompatible features which our
+	/// counterparty, or we, require.
+	//TODO: split between PeerUnconnectable/PeerDisconnected ?
+	DisconnectedPeer,
+	/// Closure generated from `ChannelManager::read` if the ChannelMonitor is newer than
+	/// the ChannelManager deserialized.
+	OutdatedChannelManager
+}
+
+impl_writeable_tlv_based_enum_upgradable!(ClosureReason,
+	(0, CounterpartyForceClosed) => { (1, peer_msg, required) },
+	(2, HolderForceClosed) => {},
+	(6, CommitmentTxConfirmed) => {},
+	(4, CooperativeClosure) => {},
+	(8, ProcessingError) => { (1, err, required) },
+	(10, DisconnectedPeer) => {},
+	(12, OutdatedChannelManager) => {},
+);
+
 /// An Event which you should probably take some action in response to.
 ///
 /// Note that while Writeable and Readable are implemented for Event, you probably shouldn't use
@@ -63,40 +160,19 @@ pub enum Event {
 	PaymentReceived {
 		/// The hash for which the preimage should be handed to the ChannelManager.
 		payment_hash: PaymentHash,
-		/// The preimage to the payment_hash, if the payment hash (and secret) were fetched via
-		/// [`ChannelManager::create_inbound_payment`]. If provided, this can be handed directly to
-		/// [`ChannelManager::claim_funds`].
-		///
-		/// [`ChannelManager::create_inbound_payment`]: crate::ln::channelmanager::ChannelManager::create_inbound_payment
-		/// [`ChannelManager::claim_funds`]: crate::ln::channelmanager::ChannelManager::claim_funds
-		payment_preimage: Option<PaymentPreimage>,
-		/// The "payment secret". This authenticates the sender to the recipient, preventing a
-		/// number of deanonymization attacks during the routing process.
-		/// It is provided here for your reference, however its accuracy is enforced directly by
-		/// [`ChannelManager`] using the values you previously provided to
-		/// [`ChannelManager::create_inbound_payment`] or
-		/// [`ChannelManager::create_inbound_payment_for_hash`].
-		///
-		/// [`ChannelManager`]: crate::ln::channelmanager::ChannelManager
-		/// [`ChannelManager::create_inbound_payment`]: crate::ln::channelmanager::ChannelManager::create_inbound_payment
-		/// [`ChannelManager::create_inbound_payment_for_hash`]: crate::ln::channelmanager::ChannelManager::create_inbound_payment_for_hash
-		payment_secret: PaymentSecret,
 		/// The value, in thousandths of a satoshi, that this payment is for. Note that you must
 		/// compare this to the expected value before accepting the payment (as otherwise you are
 		/// providing proof-of-payment for less than the value you expected!).
 		amt: u64,
-		/// This is the `user_payment_id` which was provided to
-		/// [`ChannelManager::create_inbound_payment_for_hash`] or
-		/// [`ChannelManager::create_inbound_payment`]. It has no meaning inside of LDK and is
-		/// simply copied here. It may be used to correlate PaymentReceived events with invoice
-		/// metadata stored elsewhere.
-		///
-		/// [`ChannelManager::create_inbound_payment`]: crate::ln::channelmanager::ChannelManager::create_inbound_payment
-		/// [`ChannelManager::create_inbound_payment_for_hash`]: crate::ln::channelmanager::ChannelManager::create_inbound_payment_for_hash
-		user_payment_id: u64,
+		/// Information for claiming this received payment, based on whether the purpose of the
+		/// payment is to pay an invoice or to send a spontaneous payment.
+		purpose: PaymentPurpose,
 	},
-	/// Indicates an outbound payment we made succeeded (ie it made it all the way to its target
+	/// Indicates an outbound payment we made succeeded (i.e. it made it all the way to its target
 	/// and we got back the payment preimage for it).
+	///
+	/// Note for MPP payments: in rare cases, this event may be preceded by a `PaymentPathFailed`
+	/// event. In this situation, you SHOULD treat this payment as having succeeded.
 	PaymentSent {
 		/// The preimage to the hash given to ChannelManager::send_payment.
 		/// Note that this serves as a payment receipt, if you wish to have such a thing, you must
@@ -105,13 +181,28 @@ pub enum Event {
 	},
 	/// Indicates an outbound payment we made failed. Probably some intermediary node dropped
 	/// something. You may wish to retry with a different route.
-	PaymentFailed {
+	PaymentPathFailed {
 		/// The hash which was given to ChannelManager::send_payment.
 		payment_hash: PaymentHash,
 		/// Indicates the payment was rejected for some reason by the recipient. This implies that
 		/// the payment has failed, not just the route in question. If this is not set, you may
 		/// retry the payment via a different route.
 		rejected_by_dest: bool,
+		/// Any failure information conveyed via the Onion return packet by a node along the failed
+		/// payment route.
+		///
+		/// Should be applied to the [`NetworkGraph`] so that routing decisions can take into
+		/// account the update. [`NetGraphMsgHandler`] is capable of doing this.
+		///
+		/// [`NetworkGraph`]: crate::routing::network_graph::NetworkGraph
+		/// [`NetGraphMsgHandler`]: crate::routing::network_graph::NetGraphMsgHandler
+		network_update: Option<NetworkUpdate>,
+		/// For both single-path and multi-path payments, this is set if all paths of the payment have
+		/// failed. This will be set to false if (1) this is an MPP payment and (2) other parts of the
+		/// larger MPP payment were still in flight when this event was generated.
+		all_paths_failed: bool,
+		/// The payment path that failed.
+		path: Vec<RouteHop>,
 #[cfg(test)]
 		error_code: Option<u16>,
 #[cfg(test)]
@@ -135,35 +226,76 @@ pub enum Event {
 		/// The outputs which you should store as spendable by you.
 		outputs: Vec<SpendableOutputDescriptor>,
 	},
+	/// This event is generated when a payment has been successfully forwarded through us and a
+	/// forwarding fee earned.
+	PaymentForwarded {
+		/// The fee, in milli-satoshis, which was earned as a result of the payment.
+		///
+		/// Note that if we force-closed the channel over which we forwarded an HTLC while the HTLC
+		/// was pending, the amount the next hop claimed will have been rounded down to the nearest
+		/// whole satoshi. Thus, the fee calculated here may be higher than expected as we still
+		/// claimed the full value in millisatoshis from the source. In this case,
+		/// `claim_from_onchain_tx` will be set.
+		///
+		/// If the channel which sent us the payment has been force-closed, we will claim the funds
+		/// via an on-chain transaction. In that case we do not yet know the on-chain transaction
+		/// fees which we will spend and will instead set this to `None`. It is possible duplicate
+		/// `PaymentForwarded` events are generated for the same payment iff `fee_earned_msat` is
+		/// `None`.
+		fee_earned_msat: Option<u64>,
+		/// If this is `true`, the forwarded HTLC was claimed by our counterparty via an on-chain
+		/// transaction.
+		claim_from_onchain_tx: bool,
+	},
+	/// Used to indicate that a channel with the given `channel_id` is in the process of closure.
+	ChannelClosed  {
+		/// The channel_id of the channel which has been closed. Note that on-chain transactions
+		/// resolving the channel are likely still awaiting confirmation.
+		channel_id: [u8; 32],
+		/// The reason the channel was closed.
+		reason: ClosureReason
+	}
 }
 
 impl Writeable for Event {
-	fn write<W: Writer>(&self, writer: &mut W) -> Result<(), ::std::io::Error> {
+	fn write<W: Writer>(&self, writer: &mut W) -> Result<(), io::Error> {
 		match self {
 			&Event::FundingGenerationReady { .. } => {
 				0u8.write(writer)?;
 				// We never write out FundingGenerationReady events as, upon disconnection, peers
 				// drop any channels which have not yet exchanged funding_signed.
 			},
-			&Event::PaymentReceived { ref payment_hash, ref payment_preimage, ref payment_secret, ref amt, ref user_payment_id } => {
+			&Event::PaymentReceived { ref payment_hash, ref amt, ref purpose } => {
 				1u8.write(writer)?;
+				let mut payment_secret = None;
+				let mut user_payment_id = None;
+				let payment_preimage;
+				match &purpose {
+					PaymentPurpose::InvoicePayment { payment_preimage: preimage, payment_secret: secret, user_payment_id: id } => {
+						payment_secret = Some(secret);
+						payment_preimage = *preimage;
+						user_payment_id = Some(id);
+					},
+					PaymentPurpose::SpontaneousPayment(preimage) => {
+						payment_preimage = Some(*preimage);
+					}
+				}
 				write_tlv_fields!(writer, {
-					(0, payment_hash),
-					(2, payment_secret),
-					(4, amt),
-					(6, user_payment_id),
-				}, {
-					(8, payment_preimage),
+					(0, payment_hash, required),
+					(2, payment_secret, option),
+					(4, amt, required),
+					(6, user_payment_id, option),
+					(8, payment_preimage, option),
 				});
 			},
 			&Event::PaymentSent { ref payment_preimage } => {
 				2u8.write(writer)?;
 				write_tlv_fields!(writer, {
-					(0, payment_preimage),
-				}, {});
-				payment_preimage.write(writer)?;
+					(0, payment_preimage, required),
+				});
 			},
-			&Event::PaymentFailed { ref payment_hash, ref rejected_by_dest,
+			&Event::PaymentPathFailed { ref payment_hash, ref rejected_by_dest, ref network_update,
+			                            ref all_paths_failed, ref path,
 				#[cfg(test)]
 				ref error_code,
 				#[cfg(test)]
@@ -175,51 +307,81 @@ impl Writeable for Event {
 				#[cfg(test)]
 				error_data.write(writer)?;
 				write_tlv_fields!(writer, {
-					(0, payment_hash),
-					(2, rejected_by_dest),
-				}, {});
+					(0, payment_hash, required),
+					(1, network_update, option),
+					(2, rejected_by_dest, required),
+					(3, all_paths_failed, required),
+					(5, path, vec_type),
+				});
 			},
 			&Event::PendingHTLCsForwardable { time_forwardable: _ } => {
 				4u8.write(writer)?;
-				write_tlv_fields!(writer, {}, {});
+				write_tlv_fields!(writer, {});
 				// We don't write the time_fordwardable out at all, as we presume when the user
 				// deserializes us at least that much time has elapsed.
 			},
 			&Event::SpendableOutputs { ref outputs } => {
 				5u8.write(writer)?;
 				write_tlv_fields!(writer, {
-					(0, VecWriteWrapper(outputs)),
-				}, {});
+					(0, VecWriteWrapper(outputs), required),
+				});
 			},
+			&Event::PaymentForwarded { fee_earned_msat, claim_from_onchain_tx } => {
+				7u8.write(writer)?;
+				write_tlv_fields!(writer, {
+					(0, fee_earned_msat, option),
+					(2, claim_from_onchain_tx, required),
+				});
+			},
+			&Event::ChannelClosed { ref channel_id, ref reason } => {
+				9u8.write(writer)?;
+				write_tlv_fields!(writer, {
+					(0, channel_id, required),
+					(2, reason, required)
+				});
+			},
+			// Note that, going forward, all new events must only write data inside of
+			// `write_tlv_fields`. Versions 0.0.101+ will ignore odd-numbered events that write
+			// data via `write_tlv_fields`.
 		}
 		Ok(())
 	}
 }
 impl MaybeReadable for Event {
-	fn read<R: ::std::io::Read>(reader: &mut R) -> Result<Option<Self>, msgs::DecodeError> {
+	fn read<R: io::Read>(reader: &mut R) -> Result<Option<Self>, msgs::DecodeError> {
 		match Readable::read(reader)? {
+			// Note that we do not write a length-prefixed TLV for FundingGenerationReady events,
+			// unlike all other events, thus we return immediately here.
 			0u8 => Ok(None),
 			1u8 => {
 				let f = || {
 					let mut payment_hash = PaymentHash([0; 32]);
 					let mut payment_preimage = None;
-					let mut payment_secret = PaymentSecret([0; 32]);
+					let mut payment_secret = None;
 					let mut amt = 0;
-					let mut user_payment_id = 0;
+					let mut user_payment_id = None;
 					read_tlv_fields!(reader, {
-						(0, payment_hash),
-						(2, payment_secret),
-						(4, amt),
-						(6, user_payment_id),
-					}, {
-						(8, payment_preimage),
+						(0, payment_hash, required),
+						(2, payment_secret, option),
+						(4, amt, required),
+						(6, user_payment_id, option),
+						(8, payment_preimage, option),
 					});
+					let purpose = match payment_secret {
+						Some(secret) => PaymentPurpose::InvoicePayment {
+							payment_preimage,
+							payment_secret: secret,
+							user_payment_id: if let Some(id) = user_payment_id {
+								id
+							} else { return Err(msgs::DecodeError::InvalidValue) }
+						},
+						None if payment_preimage.is_some() => PaymentPurpose::SpontaneousPayment(payment_preimage.unwrap()),
+						None => return Err(msgs::DecodeError::InvalidValue),
+					};
 					Ok(Some(Event::PaymentReceived {
 						payment_hash,
-						payment_preimage,
-						payment_secret,
 						amt,
-						user_payment_id,
+						purpose,
 					}))
 				};
 				f()
@@ -228,8 +390,8 @@ impl MaybeReadable for Event {
 				let f = || {
 					let mut payment_preimage = PaymentPreimage([0; 32]);
 					read_tlv_fields!(reader, {
-						(0, payment_preimage),
-					}, {});
+						(0, payment_preimage, required),
+					});
 					Ok(Some(Event::PaymentSent {
 						payment_preimage,
 					}))
@@ -244,13 +406,22 @@ impl MaybeReadable for Event {
 					let error_data = Readable::read(reader)?;
 					let mut payment_hash = PaymentHash([0; 32]);
 					let mut rejected_by_dest = false;
+					let mut network_update = None;
+					let mut all_paths_failed = Some(true);
+					let mut path: Option<Vec<RouteHop>> = Some(vec![]);
 					read_tlv_fields!(reader, {
-						(0, payment_hash),
-						(2, rejected_by_dest),
-					}, {});
-					Ok(Some(Event::PaymentFailed {
+						(0, payment_hash, required),
+						(1, network_update, ignorable),
+						(2, rejected_by_dest, required),
+						(3, all_paths_failed, option),
+						(5, path, vec_type),
+					});
+					Ok(Some(Event::PaymentPathFailed {
 						payment_hash,
 						rejected_by_dest,
+						network_update,
+						all_paths_failed: all_paths_failed.unwrap(),
+						path: path.unwrap(),
 						#[cfg(test)]
 						error_code,
 						#[cfg(test)]
@@ -261,7 +432,7 @@ impl MaybeReadable for Event {
 			},
 			4u8 => {
 				let f = || {
-					read_tlv_fields!(reader, {}, {});
+					read_tlv_fields!(reader, {});
 					Ok(Some(Event::PendingHTLCsForwardable {
 						time_forwardable: Duration::from_secs(0)
 					}))
@@ -272,12 +443,48 @@ impl MaybeReadable for Event {
 				let f = || {
 					let mut outputs = VecReadWrapper(Vec::new());
 					read_tlv_fields!(reader, {
-						(0, outputs),
-					}, {});
+						(0, outputs, required),
+					});
 					Ok(Some(Event::SpendableOutputs { outputs: outputs.0 }))
 				};
 				f()
 			},
+			7u8 => {
+				let f = || {
+					let mut fee_earned_msat = None;
+					let mut claim_from_onchain_tx = false;
+					read_tlv_fields!(reader, {
+						(0, fee_earned_msat, option),
+						(2, claim_from_onchain_tx, required),
+					});
+					Ok(Some(Event::PaymentForwarded { fee_earned_msat, claim_from_onchain_tx }))
+				};
+				f()
+			},
+			9u8 => {
+				let mut channel_id = [0; 32];
+				let mut reason = None;
+				read_tlv_fields!(reader, {
+					(0, channel_id, required),
+					(2, reason, ignorable),
+				});
+				if reason.is_none() { return Ok(None); }
+				Ok(Some(Event::ChannelClosed { channel_id, reason: reason.unwrap() }))
+			},
+			// Versions prior to 0.0.100 did not ignore odd types, instead returning InvalidValue.
+			// Version 0.0.100 failed to properly ignore odd types, possibly resulting in corrupt
+			// reads.
+			x if x % 2 == 1 => {
+				// If the event is of unknown type, assume it was written with `write_tlv_fields`,
+				// which prefixes the whole thing with a length BigSize. Because the event is
+				// odd-type unknown, we should treat it as `Ok(None)` even if it has some TLV
+				// fields that are even. Thus, we avoid using `read_tlv_fields` and simply read
+				// exactly the number of bytes specified, ignoring them entirely.
+				let tlv_len: BigSize = Readable::read(reader)?;
+				FixedLengthReader::new(reader, tlv_len.0)
+					.eat_remaining().map_err(|_| msgs::DecodeError::ShortRead)?;
+				Ok(None)
+			},
 			_ => Err(msgs::DecodeError::InvalidValue)
 		}
 	}
@@ -392,6 +599,15 @@ pub enum MessageSendEvent {
 		/// The channel_update which should be sent.
 		msg: msgs::ChannelUpdate,
 	},
+	/// Used to indicate that a channel_update should be sent to a single peer.
+	/// In contrast to [`Self::BroadcastChannelUpdate`], this is used when the channel is a
+	/// private channel and we shouldn't be informing all of our peers of channel parameters.
+	SendChannelUpdate {
+		/// The node_id of the node which should receive this message
+		node_id: PublicKey,
+		/// The channel_update which should be sent.
+		msg: msgs::ChannelUpdate,
+	},
 	/// Broadcast an error downstream to be handled
 	HandleError {
 		/// The node_id of the node which should receive this message
@@ -399,12 +615,6 @@ pub enum MessageSendEvent {
 		/// The action which should be taken.
 		action: msgs::ErrorAction
 	},
-	/// When a payment fails we may receive updates back from the hop where it failed. In such
-	/// cases this event is generated so that we can inform the network graph of this information.
-	PaymentFailureNetworkUpdate {
-		/// The channel/node update which should be sent to NetGraphMsgHandler
-		update: msgs::HTLCFailChannelUpdate,
-	},
 	/// Query a peer for channels with funding transaction UTXOs in a block range.
 	SendChannelRangeQuery {
 		/// The node_id of this message recipient
@@ -475,11 +685,11 @@ pub trait EventHandler {
 	/// Handles the given [`Event`].
 	///
 	/// See [`EventsProvider`] for details that must be considered when implementing this method.
-	fn handle_event(&self, event: Event);
+	fn handle_event(&self, event: &Event);
 }
 
-impl<F> EventHandler for F where F: Fn(Event) {
-	fn handle_event(&self, event: Event) {
+impl<F> EventHandler for F where F: Fn(&Event) {
+	fn handle_event(&self, event: &Event) {
 		self(event)
 	}
 }
