@@ -11,10 +11,10 @@
 //! as ChannelsManagers and ChannelMonitors.
 
 use prelude::*;
-use std::io::{Read, Write};
-use std::collections::HashMap;
+use io::{self, Read, Write};
+use io_extras::{copy, sink};
 use core::hash::Hash;
-use std::sync::Mutex;
+use sync::Mutex;
 use core::cmp;
 
 use bitcoin::secp256k1::Signature;
@@ -35,42 +35,35 @@ use util::byte_utils::{be48_to_array, slice_to_be48};
 /// serialization buffer size
 pub const MAX_BUF_SIZE: usize = 64 * 1024;
 
-/// A trait that is similar to std::io::Write but has one extra function which can be used to size
-/// buffers being written into.
-/// An impl is provided for any type that also impls std::io::Write which simply ignores size
-/// hints.
+/// A simplified version of std::io::Write that exists largely for backwards compatibility.
+/// An impl is provided for any type that also impls std::io::Write.
 ///
 /// (C-not exported) as we only export serialization to/from byte arrays instead
 pub trait Writer {
 	/// Writes the given buf out. See std::io::Write::write_all for more
-	fn write_all(&mut self, buf: &[u8]) -> Result<(), ::std::io::Error>;
-	/// Hints that data of the given size is about the be written. This may not always be called
-	/// prior to data being written and may be safely ignored.
-	fn size_hint(&mut self, size: usize);
+	fn write_all(&mut self, buf: &[u8]) -> Result<(), io::Error>;
 }
 
 impl<W: Write> Writer for W {
 	#[inline]
-	fn write_all(&mut self, buf: &[u8]) -> Result<(), ::std::io::Error> {
-		<Self as ::std::io::Write>::write_all(self, buf)
+	fn write_all(&mut self, buf: &[u8]) -> Result<(), io::Error> {
+		<Self as io::Write>::write_all(self, buf)
 	}
-	#[inline]
-	fn size_hint(&mut self, _size: usize) { }
 }
 
 pub(crate) struct WriterWriteAdaptor<'a, W: Writer + 'a>(pub &'a mut W);
 impl<'a, W: Writer + 'a> Write for WriterWriteAdaptor<'a, W> {
 	#[inline]
-	fn write_all(&mut self, buf: &[u8]) -> Result<(), ::std::io::Error> {
+	fn write_all(&mut self, buf: &[u8]) -> Result<(), io::Error> {
 		self.0.write_all(buf)
 	}
 	#[inline]
-	fn write(&mut self, buf: &[u8]) -> Result<usize, ::std::io::Error> {
+	fn write(&mut self, buf: &[u8]) -> Result<usize, io::Error> {
 		self.0.write_all(buf)?;
 		Ok(buf.len())
 	}
 	#[inline]
-	fn flush(&mut self) -> Result<(), ::std::io::Error> {
+	fn flush(&mut self) -> Result<(), io::Error> {
 		Ok(())
 	}
 }
@@ -78,14 +71,10 @@ impl<'a, W: Writer + 'a> Write for WriterWriteAdaptor<'a, W> {
 pub(crate) struct VecWriter(pub Vec<u8>);
 impl Writer for VecWriter {
 	#[inline]
-	fn write_all(&mut self, buf: &[u8]) -> Result<(), ::std::io::Error> {
+	fn write_all(&mut self, buf: &[u8]) -> Result<(), io::Error> {
 		self.0.extend_from_slice(buf);
 		Ok(())
 	}
-	#[inline]
-	fn size_hint(&mut self, size: usize) {
-		self.0.reserve_exact(size);
-	}
 }
 
 /// Writer that only tracks the amount of data written - useful if you need to calculate the length
@@ -93,12 +82,10 @@ impl Writer for VecWriter {
 pub(crate) struct LengthCalculatingWriter(pub usize);
 impl Writer for LengthCalculatingWriter {
 	#[inline]
-	fn write_all(&mut self, buf: &[u8]) -> Result<(), ::std::io::Error> {
+	fn write_all(&mut self, buf: &[u8]) -> Result<(), io::Error> {
 		self.0 += buf.len();
 		Ok(())
 	}
-	#[inline]
-	fn size_hint(&mut self, _size: usize) {}
 }
 
 /// Essentially std::io::Take but a bit simpler and with a method to walk the underlying stream
@@ -120,7 +107,7 @@ impl<R: Read> FixedLengthReader<R> {
 
 	#[inline]
 	pub fn eat_remaining(&mut self) -> Result<(), DecodeError> {
-		::std::io::copy(self, &mut ::std::io::sink()).unwrap();
+		copy(self, &mut sink()).unwrap();
 		if self.bytes_read != self.total_bytes {
 			Err(DecodeError::ShortRead)
 		} else {
@@ -130,7 +117,7 @@ impl<R: Read> FixedLengthReader<R> {
 }
 impl<R: Read> Read for FixedLengthReader<R> {
 	#[inline]
-	fn read(&mut self, dest: &mut [u8]) -> Result<usize, ::std::io::Error> {
+	fn read(&mut self, dest: &mut [u8]) -> Result<usize, io::Error> {
 		if self.total_bytes == self.bytes_read {
 			Ok(0)
 		} else {
@@ -159,7 +146,7 @@ impl<R: Read> ReadTrackingReader<R> {
 }
 impl<R: Read> Read for ReadTrackingReader<R> {
 	#[inline]
-	fn read(&mut self, dest: &mut [u8]) -> Result<usize, ::std::io::Error> {
+	fn read(&mut self, dest: &mut [u8]) -> Result<usize, io::Error> {
 		match self.read.read(dest) {
 			Ok(0) => Ok(0),
 			Ok(len) => {
@@ -176,7 +163,7 @@ impl<R: Read> Read for ReadTrackingReader<R> {
 /// (C-not exported) as we only export serialization to/from byte arrays instead
 pub trait Writeable {
 	/// Writes self out to the given Writer
-	fn write<W: Writer>(&self, writer: &mut W) -> Result<(), ::std::io::Error>;
+	fn write<W: Writer>(&self, writer: &mut W) -> Result<(), io::Error>;
 
 	/// Writes self out to a Vec<u8>
 	fn encode(&self) -> Vec<u8> {
@@ -207,7 +194,7 @@ pub trait Writeable {
 }
 
 impl<'a, T: Writeable> Writeable for &'a T {
-	fn write<W: Writer>(&self, writer: &mut W) -> Result<(), ::std::io::Error> { (*self).write(writer) }
+	fn write<W: Writer>(&self, writer: &mut W) -> Result<(), io::Error> { (*self).write(writer) }
 }
 
 /// A trait that various rust-lightning types implement allowing them to be read in from a Read
@@ -241,6 +228,13 @@ pub trait MaybeReadable
 	fn read<R: Read>(reader: &mut R) -> Result<Option<Self>, DecodeError>;
 }
 
+impl<T: Readable> MaybeReadable for T {
+	#[inline]
+	fn read<R: Read>(reader: &mut R) -> Result<Option<T>, DecodeError> {
+		Ok(Some(Readable::read(reader)?))
+	}
+}
+
 pub(crate) struct OptionDeserWrapper<T: Readable>(pub Option<T>);
 impl<T: Readable> Readable for OptionDeserWrapper<T> {
 	#[inline]
@@ -253,7 +247,7 @@ impl<T: Readable> Readable for OptionDeserWrapper<T> {
 pub(crate) struct VecWriteWrapper<'a, T: Writeable>(pub &'a Vec<T>);
 impl<'a, T: Writeable> Writeable for VecWriteWrapper<'a, T> {
 	#[inline]
-	fn write<W: Writer>(&self, writer: &mut W) -> Result<(), ::std::io::Error> {
+	fn write<W: Writer>(&self, writer: &mut W) -> Result<(), io::Error> {
 		for ref v in self.0.iter() {
 			v.write(writer)?;
 		}
@@ -262,15 +256,16 @@ impl<'a, T: Writeable> Writeable for VecWriteWrapper<'a, T> {
 }
 
 /// Wrapper to read elements from a given stream until it reaches the end of the stream.
-pub(crate) struct VecReadWrapper<T: Readable>(pub Vec<T>);
-impl<T: Readable> Readable for VecReadWrapper<T> {
+pub(crate) struct VecReadWrapper<T>(pub Vec<T>);
+impl<T: MaybeReadable> Readable for VecReadWrapper<T> {
 	#[inline]
 	fn read<R: Read>(mut reader: &mut R) -> Result<Self, DecodeError> {
 		let mut values = Vec::new();
 		loop {
 			let mut track_read = ReadTrackingReader::new(&mut reader);
-			match Readable::read(&mut track_read) {
-				Ok(v) => { values.push(v); },
+			match MaybeReadable::read(&mut track_read) {
+				Ok(Some(v)) => { values.push(v); },
+				Ok(None) => { },
 				// If we failed to read any bytes at all, we reached the end of our TLV
 				// stream and have simply exhausted all entries.
 				Err(ref e) if e == &DecodeError::ShortRead && !track_read.have_read => break,
@@ -284,7 +279,7 @@ impl<T: Readable> Readable for VecReadWrapper<T> {
 pub(crate) struct U48(pub u64);
 impl Writeable for U48 {
 	#[inline]
-	fn write<W: Writer>(&self, writer: &mut W) -> Result<(), ::std::io::Error> {
+	fn write<W: Writer>(&self, writer: &mut W) -> Result<(), io::Error> {
 		writer.write_all(&be48_to_array(self.0))
 	}
 }
@@ -307,7 +302,7 @@ impl Readable for U48 {
 pub(crate) struct BigSize(pub u64);
 impl Writeable for BigSize {
 	#[inline]
-	fn write<W: Writer>(&self, writer: &mut W) -> Result<(), ::std::io::Error> {
+	fn write<W: Writer>(&self, writer: &mut W) -> Result<(), io::Error> {
 		match self.0 {
 			0...0xFC => {
 				(self.0 as u8).write(writer)
@@ -371,13 +366,13 @@ macro_rules! impl_writeable_primitive {
 	($val_type:ty, $len: expr) => {
 		impl Writeable for $val_type {
 			#[inline]
-			fn write<W: Writer>(&self, writer: &mut W) -> Result<(), ::std::io::Error> {
+			fn write<W: Writer>(&self, writer: &mut W) -> Result<(), io::Error> {
 				writer.write_all(&self.to_be_bytes())
 			}
 		}
 		impl Writeable for HighZeroBytesDroppedVarInt<$val_type> {
 			#[inline]
-			fn write<W: Writer>(&self, writer: &mut W) -> Result<(), ::std::io::Error> {
+			fn write<W: Writer>(&self, writer: &mut W) -> Result<(), io::Error> {
 				// Skip any full leading 0 bytes when writing (in BE):
 				writer.write_all(&self.0.to_be_bytes()[(self.0.leading_zeros()/8) as usize..$len])
 			}
@@ -425,7 +420,7 @@ impl_writeable_primitive!(u16, 2);
 
 impl Writeable for u8 {
 	#[inline]
-	fn write<W: Writer>(&self, writer: &mut W) -> Result<(), ::std::io::Error> {
+	fn write<W: Writer>(&self, writer: &mut W) -> Result<(), io::Error> {
 		writer.write_all(&[*self])
 	}
 }
@@ -440,7 +435,7 @@ impl Readable for u8 {
 
 impl Writeable for bool {
 	#[inline]
-	fn write<W: Writer>(&self, writer: &mut W) -> Result<(), ::std::io::Error> {
+	fn write<W: Writer>(&self, writer: &mut W) -> Result<(), io::Error> {
 		writer.write_all(&[if *self {1} else {0}])
 	}
 }
@@ -462,7 +457,7 @@ macro_rules! impl_array {
 		impl Writeable for [u8; $size]
 		{
 			#[inline]
-			fn write<W: Writer>(&self, w: &mut W) -> Result<(), ::std::io::Error> {
+			fn write<W: Writer>(&self, w: &mut W) -> Result<(), io::Error> {
 				w.write_all(self)
 			}
 		}
@@ -495,7 +490,7 @@ impl<K, V> Writeable for HashMap<K, V>
 	      V: Writeable
 {
 	#[inline]
-	fn write<W: Writer>(&self, w: &mut W) -> Result<(), ::std::io::Error> {
+	fn write<W: Writer>(&self, w: &mut W) -> Result<(), io::Error> {
 	(self.len() as u16).write(w)?;
 		for (key, value) in self.iter() {
 			key.write(w)?;
@@ -520,10 +515,40 @@ impl<K, V> Readable for HashMap<K, V>
 	}
 }
 
+// HashSet
+impl<T> Writeable for HashSet<T>
+where T: Writeable + Eq + Hash
+{
+	#[inline]
+	fn write<W: Writer>(&self, w: &mut W) -> Result<(), io::Error> {
+		(self.len() as u16).write(w)?;
+		for item in self.iter() {
+			item.write(w)?;
+		}
+		Ok(())
+	}
+}
+
+impl<T> Readable for HashSet<T>
+where T: Readable + Eq + Hash
+{
+	#[inline]
+	fn read<R: Read>(r: &mut R) -> Result<Self, DecodeError> {
+		let len: u16 = Readable::read(r)?;
+		let mut ret = HashSet::with_capacity(len as usize);
+		for _ in 0..len {
+			if !ret.insert(T::read(r)?) {
+				return Err(DecodeError::InvalidValue)
+			}
+		}
+		Ok(ret)
+	}
+}
+
 // Vectors
 impl Writeable for Vec<u8> {
 	#[inline]
-	fn write<W: Writer>(&self, w: &mut W) -> Result<(), ::std::io::Error> {
+	fn write<W: Writer>(&self, w: &mut W) -> Result<(), io::Error> {
 		(self.len() as u16).write(w)?;
 		w.write_all(&self)
 	}
@@ -541,7 +566,7 @@ impl Readable for Vec<u8> {
 }
 impl Writeable for Vec<Signature> {
 	#[inline]
-	fn write<W: Writer>(&self, w: &mut W) -> Result<(), ::std::io::Error> {
+	fn write<W: Writer>(&self, w: &mut W) -> Result<(), io::Error> {
 		(self.len() as u16).write(w)?;
 		for e in self.iter() {
 			e.write(w)?;
@@ -561,13 +586,13 @@ impl Readable for Vec<Signature> {
 			return Err(DecodeError::BadLengthDescriptor);
 		}
 		let mut ret = Vec::with_capacity(len as usize);
-		for _ in 0..len { ret.push(Signature::read(r)?); }
+		for _ in 0..len { ret.push(Readable::read(r)?); }
 		Ok(ret)
 	}
 }
 
 impl Writeable for Script {
-	fn write<W: Writer>(&self, w: &mut W) -> Result<(), ::std::io::Error> {
+	fn write<W: Writer>(&self, w: &mut W) -> Result<(), io::Error> {
 		(self.len() as u16).write(w)?;
 		w.write_all(self.as_bytes())
 	}
@@ -583,7 +608,7 @@ impl Readable for Script {
 }
 
 impl Writeable for PublicKey {
-	fn write<W: Writer>(&self, w: &mut W) -> Result<(), ::std::io::Error> {
+	fn write<W: Writer>(&self, w: &mut W) -> Result<(), io::Error> {
 		self.serialize().write(w)
 	}
 	#[inline]
@@ -603,7 +628,7 @@ impl Readable for PublicKey {
 }
 
 impl Writeable for SecretKey {
-	fn write<W: Writer>(&self, w: &mut W) -> Result<(), ::std::io::Error> {
+	fn write<W: Writer>(&self, w: &mut W) -> Result<(), io::Error> {
 		let mut ser = [0; SECRET_KEY_SIZE];
 		ser.copy_from_slice(&self[..]);
 		ser.write(w)
@@ -625,7 +650,7 @@ impl Readable for SecretKey {
 }
 
 impl Writeable for Sha256dHash {
-	fn write<W: Writer>(&self, w: &mut W) -> Result<(), ::std::io::Error> {
+	fn write<W: Writer>(&self, w: &mut W) -> Result<(), io::Error> {
 		w.write_all(&self[..])
 	}
 }
@@ -640,7 +665,7 @@ impl Readable for Sha256dHash {
 }
 
 impl Writeable for Signature {
-	fn write<W: Writer>(&self, w: &mut W) -> Result<(), ::std::io::Error> {
+	fn write<W: Writer>(&self, w: &mut W) -> Result<(), io::Error> {
 		self.serialize_compact().write(w)
 	}
 	#[inline]
@@ -660,7 +685,7 @@ impl Readable for Signature {
 }
 
 impl Writeable for PaymentPreimage {
-	fn write<W: Writer>(&self, w: &mut W) -> Result<(), ::std::io::Error> {
+	fn write<W: Writer>(&self, w: &mut W) -> Result<(), io::Error> {
 		self.0.write(w)
 	}
 }
@@ -673,7 +698,7 @@ impl Readable for PaymentPreimage {
 }
 
 impl Writeable for PaymentHash {
-	fn write<W: Writer>(&self, w: &mut W) -> Result<(), ::std::io::Error> {
+	fn write<W: Writer>(&self, w: &mut W) -> Result<(), io::Error> {
 		self.0.write(w)
 	}
 }
@@ -686,7 +711,7 @@ impl Readable for PaymentHash {
 }
 
 impl Writeable for PaymentSecret {
-	fn write<W: Writer>(&self, w: &mut W) -> Result<(), ::std::io::Error> {
+	fn write<W: Writer>(&self, w: &mut W) -> Result<(), io::Error> {
 		self.0.write(w)
 	}
 }
@@ -699,7 +724,7 @@ impl Readable for PaymentSecret {
 }
 
 impl<T: Writeable> Writeable for Box<T> {
-	fn write<W: Writer>(&self, w: &mut W) -> Result<(), ::std::io::Error> {
+	fn write<W: Writer>(&self, w: &mut W) -> Result<(), io::Error> {
 		T::write(&**self, w)
 	}
 }
@@ -711,7 +736,7 @@ impl<T: Readable> Readable for Box<T> {
 }
 
 impl<T: Writeable> Writeable for Option<T> {
-	fn write<W: Writer>(&self, w: &mut W) -> Result<(), ::std::io::Error> {
+	fn write<W: Writer>(&self, w: &mut W) -> Result<(), io::Error> {
 		match *self {
 			None => 0u8.write(w)?,
 			Some(ref data) => {
@@ -726,7 +751,8 @@ impl<T: Writeable> Writeable for Option<T> {
 impl<T: Readable> Readable for Option<T>
 {
 	fn read<R: Read>(r: &mut R) -> Result<Self, DecodeError> {
-		match BigSize::read(r)?.0 {
+		let len: BigSize = Readable::read(r)?;
+		match len.0 {
 			0 => Ok(None),
 			len => {
 				let mut reader = FixedLengthReader::new(r, len - 1);
@@ -737,7 +763,7 @@ impl<T: Readable> Readable for Option<T>
 }
 
 impl Writeable for Txid {
-	fn write<W: Writer>(&self, w: &mut W) -> Result<(), ::std::io::Error> {
+	fn write<W: Writer>(&self, w: &mut W) -> Result<(), io::Error> {
 		w.write_all(&self[..])
 	}
 }
@@ -752,7 +778,7 @@ impl Readable for Txid {
 }
 
 impl Writeable for BlockHash {
-	fn write<W: Writer>(&self, w: &mut W) -> Result<(), ::std::io::Error> {
+	fn write<W: Writer>(&self, w: &mut W) -> Result<(), io::Error> {
 		w.write_all(&self[..])
 	}
 }
@@ -767,7 +793,7 @@ impl Readable for BlockHash {
 }
 
 impl Writeable for OutPoint {
-	fn write<W: Writer>(&self, w: &mut W) -> Result<(), ::std::io::Error> {
+	fn write<W: Writer>(&self, w: &mut W) -> Result<(), io::Error> {
 		self.txid.write(w)?;
 		self.vout.write(w)?;
 		Ok(())
@@ -788,7 +814,7 @@ impl Readable for OutPoint {
 macro_rules! impl_consensus_ser {
 	($bitcoin_type: ty) => {
 		impl Writeable for $bitcoin_type {
-			fn write<W: Writer>(&self, writer: &mut W) -> Result<(), ::std::io::Error> {
+			fn write<W: Writer>(&self, writer: &mut W) -> Result<(), io::Error> {
 				match self.consensus_encode(WriterWriteAdaptor(writer)) {
 					Ok(_) => Ok(()),
 					Err(e) => Err(e),
@@ -800,7 +826,7 @@ macro_rules! impl_consensus_ser {
 			fn read<R: Read>(r: &mut R) -> Result<Self, DecodeError> {
 				match consensus::encode::Decodable::consensus_decode(r) {
 					Ok(t) => Ok(t),
-					Err(consensus::encode::Error::Io(ref e)) if e.kind() == ::std::io::ErrorKind::UnexpectedEof => Err(DecodeError::ShortRead),
+					Err(consensus::encode::Error::Io(ref e)) if e.kind() == io::ErrorKind::UnexpectedEof => Err(DecodeError::ShortRead),
 					Err(consensus::encode::Error::Io(e)) => Err(DecodeError::Io(e.kind())),
 					Err(_) => Err(DecodeError::InvalidValue),
 				}
@@ -818,7 +844,7 @@ impl<T: Readable> Readable for Mutex<T> {
 	}
 }
 impl<T: Writeable> Writeable for Mutex<T> {
-	fn write<W: Writer>(&self, w: &mut W) -> Result<(), ::std::io::Error> {
+	fn write<W: Writer>(&self, w: &mut W) -> Result<(), io::Error> {
 		self.lock().unwrap().write(w)
 	}
 }
@@ -831,7 +857,7 @@ impl<A: Readable, B: Readable> Readable for (A, B) {
 	}
 }
 impl<A: Writeable, B: Writeable> Writeable for (A, B) {
-	fn write<W: Writer>(&self, w: &mut W) -> Result<(), ::std::io::Error> {
+	fn write<W: Writer>(&self, w: &mut W) -> Result<(), io::Error> {
 		self.0.write(w)?;
 		self.1.write(w)
 	}
@@ -846,9 +872,36 @@ impl<A: Readable, B: Readable, C: Readable> Readable for (A, B, C) {
 	}
 }
 impl<A: Writeable, B: Writeable, C: Writeable> Writeable for (A, B, C) {
-	fn write<W: Writer>(&self, w: &mut W) -> Result<(), ::std::io::Error> {
+	fn write<W: Writer>(&self, w: &mut W) -> Result<(), io::Error> {
 		self.0.write(w)?;
 		self.1.write(w)?;
 		self.2.write(w)
 	}
 }
+
+impl Writeable for () {
+	fn write<W: Writer>(&self, _: &mut W) -> Result<(), io::Error> {
+		Ok(())
+	}
+}
+impl Readable for () {
+	fn read<R: Read>(_r: &mut R) -> Result<Self, DecodeError> {
+		Ok(())
+	}
+}
+
+impl Writeable for String {
+	#[inline]
+	fn write<W: Writer>(&self, w: &mut W) -> Result<(), io::Error> {
+		(self.len() as u16).write(w)?;
+		w.write_all(self.as_bytes())
+	}
+}
+impl Readable for String {
+	#[inline]
+	fn read<R: Read>(r: &mut R) -> Result<Self, DecodeError> {
+		let v: Vec<u8> = Readable::read(r)?;
+		let ret = String::from_utf8(v).map_err(|_| DecodeError::InvalidValue)?;
+		Ok(ret)
+	}
+}
